@@ -2,7 +2,7 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2017 - ROLI Ltd.
+   Copyright (c) 2022 - Raw Material Software Limited
 
    JUCE is an open source library subject to commercial or open-source
    licensing.
@@ -28,11 +28,6 @@ class ThreadPoolJob;
 class ActionListener;
 class ActionBroadcaster;
 
-//==============================================================================
-#if JUCE_MODULE_AVAILABLE_juce_opengl
-class OpenGLContext;
-#endif
-
 //==============================================================================
 /** See MessageManager::callFunctionOnMessageThread() for use of this function type. */
 using MessageCallbackFunction = void* (void* userData);
@@ -84,7 +79,7 @@ public:
     */
     bool hasStopMessageBeenSent() const noexcept        { return quitMessagePosted.get() != 0; }
 
-   #if JUCE_MODAL_LOOPS_PERMITTED || DOXYGEN
+   #if JUCE_MODAL_LOOPS_PERMITTED
     /** Synchronously dispatches messages until a given time has elapsed.
 
         Returns false if a quit message has been posted by a call to stopDispatchLoop(),
@@ -208,7 +203,7 @@ public:
             Creates a new critical section to exclusively access methods which can
             only be called when the message manager is locked.
 
-            Unlike CrititcalSection, multiple instances of this lock class provide
+            Unlike CriticalSection, multiple instances of this lock class provide
             exclusive access to a single resource - the MessageManager.
         */
         Lock();
@@ -302,13 +297,23 @@ public:
         struct BlockingMessage;
         friend class ReferenceCountedObjectPtr<BlockingMessage>;
 
+        bool exclusiveTryAcquire (bool) const noexcept;
         bool tryAcquire (bool) const noexcept;
-        void messageCallback() const;
+
+        void setAcquired (bool success) const noexcept;
 
         //==============================================================================
+        // This mutex is used to make this lock type behave like a normal mutex.
+        // If multiple threads call enter() simultaneously, only one will succeed in gaining
+        // this mutex. The mutex is released again in exit().
+        mutable CriticalSection entryMutex;
+
+        // This mutex protects the other data members of the lock from concurrent access, which
+        // happens when the BlockingMessage calls setAcquired to indicate that the lock was gained.
+        mutable std::mutex mutex;
         mutable ReferenceCountedObjectPtr<BlockingMessage> blockingMessage;
-        WaitableEvent lockedEvent;
-        mutable Atomic<int> abortWait, lockGained;
+        mutable std::condition_variable condvar;
+        mutable bool abortWait = false, acquired = false;
     };
 
     //==============================================================================
@@ -333,12 +338,12 @@ private:
     Atomic<int> quitMessagePosted { 0 }, quitMessageReceived { 0 };
     Thread::ThreadID messageThreadId;
     Atomic<Thread::ThreadID> threadWithLock;
+    mutable std::mutex messageThreadIdMutex;
 
     static bool postMessageToSystemQueue (MessageBase*);
     static void* exitModalLoopCallback (void*);
     static void doPlatformSpecificInitialisation();
     static void doPlatformSpecificShutdown();
-    static bool dispatchNextMessageOnSystemQueue (bool returnIfNoPendingMessages);
 
     JUCE_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR (MessageManager)
 };
