@@ -35,12 +35,12 @@
 
   ID:                 juce_graphics
   vendor:             juce
-  version:            7.0.2
+  version:            7.0.12
   name:               JUCE graphics classes
   description:        Classes for 2D vector graphics, image loading/saving, font handling, etc.
   website:            http://www.juce.com/juce
   license:            GPL/Commercial
-  minimumCppStandard: 14
+  minimumCppStandard: 17
 
   dependencies:       juce_events
   OSXFrameworks:      Cocoa QuartzCore
@@ -150,10 +150,10 @@ namespace juce
 #include "effects/juce_GlowEffect.h"
 
 #if JUCE_GRAPHICS_INCLUDE_COREGRAPHICS_HELPERS && (JUCE_MAC || JUCE_IOS)
- #include "native/juce_mac_CoreGraphicsHelpers.h"
- #include "native/juce_mac_CoreGraphicsContext.h"
+ #include "native/juce_CoreGraphicsHelpers_mac.h"
+ #include "native/juce_CoreGraphicsContext_mac.h"
 #endif
 
 #if JUCE_DIRECT2D && JUCE_WINDOWS
-#include "native/juce_win32_Direct2DGraphicsContext.h"
+#include "native/juce_Direct2DGraphicsContext_windows.h"
 #endif
