@@ -2,15 +2,15 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2020 - Raw Material Software Limited
+   Copyright (c) 2022 - Raw Material Software Limited
 
    JUCE is an open source library subject to commercial or open-source
    licensing.
 
-   By using JUCE, you agree to the terms of both the JUCE 6 End-User License
-   Agreement and JUCE Privacy Policy (both effective as of the 16th June 2020).
+   By using JUCE, you agree to the terms of both the JUCE 7 End-User License
+   Agreement and JUCE Privacy Policy.
 
-   End User License Agreement: www.juce.com/juce-6-licence
+   End User License Agreement: www.juce.com/juce-7-licence
    Privacy Policy: www.juce.com/juce-privacy-policy
 
    Or: You may also use this code under the terms of the GPL v3 (see
@@ -26,10 +26,10 @@
 namespace juce
 {
 
-struct Button::CallbackHelper  : public Timer,
-                                 public ApplicationCommandManagerListener,
-                                 public Value::Listener,
-                                 public KeyListener
+struct Button::CallbackHelper final : public Timer,
+                                      public ApplicationCommandManagerListener,
+                                      public Value::Listener,
+                                      public KeyListener
 {
     CallbackHelper (Button& b) : button (b)   {}
 
@@ -122,7 +122,7 @@ void Button::updateAutomaticTooltip (const ApplicationCommandInfo& info)
             tt << " [";
 
             if (key.length() == 1)
-                tt << TRANS("shortcut") << ": '" << key << "']";
+                tt << TRANS ("shortcut") << ": '" << key << "']";
             else
                 tt << key << ']';
         }
@@ -141,6 +141,20 @@ void Button::setConnectedEdges (int newFlags)
 }
 
 //==============================================================================
+void Button::checkToggleableState (bool wasToggleable)
+{
+    if (isToggleable() != wasToggleable)
+        invalidateAccessibilityHandler();
+}
+
+void Button::setToggleable (bool isNowToggleable)
+{
+    const auto wasToggleable = isToggleable();
+
+    canBeToggled = isNowToggleable;
+    checkToggleableState (wasToggleable);
+}
+
 void Button::setToggleState (bool shouldBeOn, NotificationType notification)
 {
     setToggleState (shouldBeOn, notification, notification);
@@ -201,7 +215,10 @@ void Button::setToggleState (bool shouldBeOn, bool sendChange)
 
 void Button::setClickingTogglesState (bool shouldToggle) noexcept
 {
+    const auto wasToggleable = isToggleable();
+
     clickTogglesState = shouldToggle;
+    checkToggleableState (wasToggleable);
 
     // if you've got clickTogglesState turned on, you shouldn't also connect the button
     // up to be a command invoker. Instead, your command handler must flip the state of whatever
@@ -210,11 +227,6 @@ void Button::setClickingTogglesState (bool shouldToggle) noexcept
     jassert (commandManagerToUse == nullptr || ! clickTogglesState);
 }
 
-bool Button::getClickingTogglesState() const noexcept
-{
-    return clickTogglesState;
-}
-
 void Button::setRadioGroupId (int newGroupId, NotificationType notification)
 {
     if (radioGroupId != newGroupId)
@@ -224,6 +236,7 @@ void Button::setRadioGroupId (int newGroupId, NotificationType notification)
         if (lastToggleState)
             turnOffOtherButtonsInGroup (notification, notification);
 
+        setToggleable (true);
         invalidateAccessibilityHandler();
     }
 }
@@ -408,8 +421,7 @@ void Button::sendClickMessage (const ModifierKeys& modifiers)
     if (checker.shouldBailOut())
         return;
 
-    if (onClick != nullptr)
-        onClick();
+    NullCheckedInvocation::invoke (onClick);
 }
 
 void Button::sendStateMessage()
@@ -426,8 +438,7 @@ void Button::sendStateMessage()
     if (checker.shouldBailOut())
         return;
 
-    if (onStateChange != nullptr)
-        onStateChange();
+    NullCheckedInvocation::invoke (onStateChange);
 }
 
 //==============================================================================
@@ -702,77 +713,9 @@ void Button::repeatTimerCallback()
     }
 }
 
-//==============================================================================
-class ButtonAccessibilityHandler  : public AccessibilityHandler
-{
-public:
-    explicit ButtonAccessibilityHandler (Button& buttonToWrap, AccessibilityRole roleIn)
-        : AccessibilityHandler (buttonToWrap,
-                                isRadioButton (buttonToWrap) ? AccessibilityRole::radioButton : roleIn,
-                                getAccessibilityActions (buttonToWrap, roleIn)),
-          button (buttonToWrap)
-    {
-    }
-
-    AccessibleState getCurrentState() const override
-    {
-        auto state = AccessibilityHandler::getCurrentState();
-
-        if (isToggleButton (getRole()) || isRadioButton (button))
-        {
-            state = state.withCheckable();
-
-            if (button.getToggleState())
-                state = state.withChecked();
-        }
-
-        return state;
-    }
-
-    String getTitle() const override
-    {
-        auto title = AccessibilityHandler::getTitle();
-
-        if (title.isEmpty())
-            return button.getButtonText();
-
-        return title;
-    }
-
-    String getHelp() const override  { return button.getTooltip(); }
-
-private:
-    static bool isToggleButton (AccessibilityRole role) noexcept
-    {
-        return role == AccessibilityRole::toggleButton;
-    }
-
-    static bool isRadioButton (const Button& button) noexcept
-    {
-        return button.getRadioGroupId() != 0;
-    }
-
-    static AccessibilityActions getAccessibilityActions (Button& button, AccessibilityRole role)
-    {
-        auto actions = AccessibilityActions().addAction (AccessibilityActionType::press,
-                                                         [&button] { button.triggerClick(); });
-
-        if (isToggleButton (role))
-            actions = actions.addAction (AccessibilityActionType::toggle,
-                                         [&button] { button.setToggleState (! button.getToggleState(), sendNotification); });
-
-        return actions;
-    }
-
-    Button& button;
-
-    //==============================================================================
-    JUCE_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR (ButtonAccessibilityHandler)
-};
-
 std::unique_ptr<AccessibilityHandler> Button::createAccessibilityHandler()
 {
-    return std::make_unique<ButtonAccessibilityHandler> (*this, AccessibilityRole::button);
+    return std::make_unique<detail::ButtonAccessibilityHandler> (*this, AccessibilityRole::button);
 }
 
 } // namespace juce
